import { readJSON as readJSONbase } from 'fs-extra';
import type { PackageJson as PackageJsonBase, TsConfigJson as TsConfigJsonBase } from 'type-fest';
import findUp from 'find-up';
import globby from 'globby';
import { WriteFileOptions } from './format/types';
export { findUp, globby };
export { existsSync, exists, ensureDir, ensureDirSync, emptyDirSync, emptyDir, copy, copySync, move, moveSync, remove, removeSync, } from 'fs-extra';
export interface PackageJson extends PackageJsonBase, Record<string, unknown> {
    scripts?: {
        [scriptName: string]: string;
    };
    exports?: {
        [key: string]: PackageJsonBase.Exports;
    };
}
export interface TsConfigJson extends TsConfigJsonBase, Record<string, unknown> {
}
export declare function writeFile({ path, contents, isBase64, }: WriteFileOptions): Promise<void>;
export declare function writeFileSync({ path, contents, isBase64, }: WriteFileOptions): void;
export declare function writeFormattedFile({ path, contents, }: WriteFileOptions): Promise<void>;
export declare function writeFormattedFileSync({ path, contents, }: WriteFileOptions): void;
/**
 * @see {writeFormattedFile} if you want Prettier formatting
 */
export declare const writeJSON: typeof writeFile;
/**
 * @see {writeFormattedFileSync} if you want Prettier formatting
 */
export declare const writeJSONSync: typeof writeFileSync;
export declare function readJSON<T extends Record<string, unknown>>(path: string, options?: Parameters<typeof readJSONbase>[1]): Promise<T>;
export declare function readJSONSync<T extends Record<string, unknown>>(path: string, options?: Parameters<typeof readJSONbase>[1]): T;
export declare function readFile(path: string): Promise<string>;
export declare function readFileSync(path: string): string;
export declare function parseYaml<T>(yaml: string): T;
export declare function stringifyYaml(yaml: Record<string, unknown>, options?: {
    doubleQuote?: boolean;
}): string;
export declare function readYaml<T extends Record<string, unknown>>(path: string): Promise<T>;
export declare function readYamlSync<T extends Record<string, unknown>>(path: string): T;
export declare function writeYaml({ path, contents, }: {
    path: string;
    contents: Record<string, unknown>;
}): Promise<void>;
export declare function writeYamlSync({ path, contents, }: {
    path: string;
    contents: Record<string, unknown>;
}): void;
export declare function writeYamlFormatted({ path, contents, }: {
    path: string;
    contents: Record<string, unknown>;
}): Promise<void>;
/** Type-safe package.json reader */
export declare function getPackageJsonSync(pkgPath: string): PackageJson;
/**
 * Ensures a directory exists. If it does not exist, it will be created.
 * If it's been ensured before, it will be skipped.
 * Use `ensureDir` if there's a chance it was ensured and then deleted
 */
export declare function ensureDirCached(dir: string): Promise<void>;
/** Type-safe package.json reader */
export declare function getModulePkgJson(moduleName: string | {
    moduleName: string;
    resolveFrom?: string;
}): PackageJson;
export declare function findUpPkgJson(
/**
 * Directory to start looking from.
 * Usually `__dirname` or `process.cwd()`
 * Ok for path to end in `package.json`
 */
dir: string): {
    pkgPath: string;
    pkgDir: string;
    pkg: PackageJson;
    exists: boolean;
};
export declare function findUpPkgJsonAsync(
/**
 * Directory to start looking from.
 * Usually `__dirname` or `process.cwd()`
 * Ok for path to end in `package.json`
 */
dir: string): Promise<{
    pkgPath: string;
    pkgDir: string;
    pkg: PackageJson;
    exists: boolean;
}>;
export declare function hasFileUp({ file, cwd, }: {
    /** Either filename or list of possibilities like `['.babelrc', '.babelrc.json']` */
    file: string | string[];
    cwd?: string;
}): boolean;
export declare function assertFileExists(filePath: string, errorMsg?: string): void;
export declare function findAllParentPkgJsons(dir: string): {
    pkgPath: string;
    dir: string;
}[];
export declare function findGitRoot(dir?: string): string;
/**
 * If a package.json is found in a directory that is NOT the git root, then this returns true
 */
export declare function isMonorepoPkg(dir?: string): boolean;
/**
 * Attempts to find the "root" directory (not `/`). Looks in this order:
 * 1) Git repo root
 * 2) A package.json that is highest up in directories
 * 3) The directory parameter passed into this function, which defaults to the CWD
 */
export declare function findRootDir(dir?: string): string;
/** Type-safe package.json reader */
export declare function getPackageJson(pkgPath: string): Promise<PackageJson>;
export declare function updatePkgJson({ pkg, dir, throwErrorIfAbsent, }: {
    /** Either path to directory with `package.json` OR full path to `package.json` */
    dir: string;
    /** will be deep merged into `package.json` */
    pkg: Partial<PackageJson>;
    throwErrorIfAbsent?: boolean;
}): Promise<{
    pkg: PackageJson;
    isNew: boolean;
    previousPkg?: PackageJson;
}>;
export declare function updateFile({ filePath, alterFile, }: {
    filePath: string;
    alterFile: (contents: string) => string | Promise<string>;
}): Promise<void>;
export declare function updateJsonFile<Data extends Record<string, unknown> = Record<string, unknown>>({ filePath, alterFile, }: {
    filePath: string;
    alterFile: (contents: Data) => Data | Promise<Data>;
}): Promise<void>;
interface Folder {
    [key: string]: Folder | string | string[] | false;
}
/**
 * Use a nested object to write a folder of files easily
 * @example
 * writeFolderOfFiles({
 *   dir: process.cwd(),
 *   handlePriorFile: 'skip',
 *   files: {
 *     'readme.md': '# Welcome!',
 *     'file-to-skip.txt': false,
 *     src: {
 *       'index.ts': 'export const hi = "hello";',
 *       'bigger-file.ts': ['first line', 'second line'],
 *     },
 *   },
 * })
 */
export declare function writeFolderOfFiles({ dir, files, handlePriorFile, }: {
    dir?: string;
    files: Folder;
    handlePriorFile: 'overwrite' | 'skip' | 'throwError';
}): Promise<void>;
export declare function isPathDir(path: string): boolean;
export declare function isPathFile(path: string): boolean;
export declare function isPathSymlink(path: string): boolean;
export declare function assertDirExists(dir: string): void;
export declare function isMonorepo(): boolean;
export declare function getMonorepoPkgs(): {
    pkgs: {
        pkgJson: PackageJson;
        dir: string;
    }[];
    root: string;
    type: "npm" | "lerna" | "pnpm" | "yarn";
};
//# sourceMappingURL=file-utils.d.ts.map