"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getJsExportNames = void 0;
const os_1 = require("os");
const path_1 = require("path");
const resolve_1 = require("./resolve");
const [nodeVer] = process.versions.node.split('.');
const baseDir = (0, path_1.resolve)((0, os_1.tmpdir)(), 'ks-import-js-utils');
async function getJsExportNames({ path, define, alias, resolveFromDir = process.cwd(), }) {
    const { build } = await import('esbuild');
    const { absolutePath, exists, type } = await (0, resolve_1.resolvePath)({
        path,
        resolveFromDir,
    });
    if (!exists) {
        throw new Error(`Cannot find path to import "${path}"`);
    }
    const pathToImport = type === 'package' ? path : absolutePath;
    const file = (0, path_1.join)(baseDir, `x.js`);
    try {
        const res = await build({
            entryPoints: [pathToImport],
            target: `node${nodeVer}`,
            platform: 'browser',
            format: 'esm',
            bundle: true,
            absWorkingDir: resolveFromDir,
            outfile: file,
            splitting: false,
            // https://esbuild.github.io/api/#write
            // IMPORTANT: we don't actually write the output, because we are just interested in the metafile
            write: false,
            jsx: 'transform',
            logLevel: 'silent',
            sourcemap: false,
            drop: ['console', 'debugger'],
            // https://esbuild.github.io/api/#ignore-annotations
            // ignore `sideEffects: false` in package.json
            ignoreAnnotations: true,
            // https://esbuild.github.io/api/#metafile
            metafile: true,
            define,
            alias,
            loader: Object.fromEntries(
            // all of these extensions will use the "empty" loader, which means they will be ignored
            [
                '.png',
                '.svg',
                '.jpg',
                '.jpeg',
                '.gql',
                '.graphql',
                '.md',
                '.mdx',
                '.css',
                '.scss',
                '.sass',
                '.less',
            ].map((ext) => [ext, 'empty'])),
        });
        const fileInfo = res.metafile.outputs[(0, path_1.relative)(resolveFromDir, file)];
        if (!fileInfo) {
            return {
                exports: [],
                errorMsg: `Error building before importing JsModule ${path}: no metafile found`,
            };
        }
        if (Array.isArray(fileInfo.exports)) {
            // if we got the array of exports, then we're good - no need to look at potential errors below
            return { exports: fileInfo.exports };
        }
        if (res.errors.length > 0) {
            // perhaps try using `formatMessages` to format the error messages? says it's for terminal output though
            const msgs = res.errors.map(({ text }) => text).join('\n');
            throw new Error(msgs);
        }
        // if we got here, then we didn't get the array of exports, and we didn't get any errors
        return {
            exports: [],
        };
    }
    catch (e) {
        return {
            exports: [],
            errorMsg: `Error building before importing JsModule ${path}: ${e.message}`,
        };
    }
}
exports.getJsExportNames = getJsExportNames;
//# sourceMappingURL=get-js-exports.js.map