"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolvePath = exports.resolvePathSync = exports.resolvePkgExport = exports.convertJsExtensionToDts = exports.getPathType = void 0;
const resolve_1 = __importDefault(require("resolve"));
const resolve_exports_1 = require("resolve.exports");
const node_path_1 = require("node:path");
const file_utils_1 = require("./file-utils");
/** Time in ms to keep the resolve path cache (sync & async) */
const resolvePathCacheLife = 10000;
function getPathType(path) {
    if (typeof path !== 'string') {
        throw new Error('Path must be a string');
    }
    if (path.length === 0) {
        throw new Error('Path cannot be an empty string');
    }
    if ((0, node_path_1.isAbsolute)(path)) {
        return {
            type: 'absolute',
        };
    }
    if (path.startsWith('.')) {
        return {
            type: 'relative',
        };
    }
    const isScoppedPkg = path.startsWith('@');
    const pieces = path.split('/');
    const totalSlashes = pieces.length;
    const isSubPath = isScoppedPkg ? totalSlashes > 2 : totalSlashes > 1;
    if (!isSubPath) {
        return {
            type: 'package',
        };
    }
    const pkgName = pieces.slice(0, isScoppedPkg ? 2 : 1).join('/');
    const subPath = `./${pieces.slice(isScoppedPkg ? 2 : 1).join('/')}`;
    return {
        type: 'package-sub-path',
        pkgName,
        subPath,
    };
}
exports.getPathType = getPathType;
function convertJsExtensionToDts(jsExt) {
    if (!jsExt.startsWith('.')) {
        throw new Error('Extension must start with a "."');
    }
    switch (jsExt) {
        case '.json':
            return '.d.ts';
        default:
            return jsExt.replace('.', '.d.').replace('j', 't').replace('x', '');
    }
}
exports.convertJsExtensionToDts = convertJsExtensionToDts;
function getConditions(resolveType) {
    switch (resolveType) {
        case 'mjs':
            return ['module', 'import', 'browser', 'default'];
        case 'cjs':
            return ['require', 'default'];
        case 'types':
            return ['types'];
        default: {
            const _ = resolveType;
        }
    }
}
function resolvePkgExport({ pkg, resolveType, subPath = '.', }) {
    if (pkg.exports) {
        try {
            const results = (0, resolve_exports_1.exports)(pkg, subPath, {
                // makes it so all `conditions` must be explicit - besides `default`
                unsafe: true,
                // warning, no matter what the `default` condition will apply
                conditions: getConditions(resolveType),
            });
            if (results && results.length > 0) {
                const [result] = results;
                if (resolveType === 'types' && !result.includes('d.ts')) {
                    // if the result is not a `d.ts` file, it's likely the `default` condition applied
                    if (subPath === '.') {
                        // only since this is the package root, check for `types` and `typings`
                        if (pkg.types)
                            return pkg.types;
                        if (pkg.typings)
                            return pkg.typings;
                    }
                }
                else {
                    return result;
                }
            }
        }
        catch (e) {
            return undefined;
        }
    }
    // the below only applies for package root
    if (subPath !== '.')
        return;
    switch (resolveType) {
        case 'mjs': {
            if (pkg.module)
                return pkg.module;
            break;
        }
        case 'types': {
            if (pkg.types)
                return pkg.types;
            if (pkg.typings)
                return pkg.typings;
            break;
        }
        case 'cjs': {
            return pkg.main;
        }
        default: {
            const _ = resolveType;
        }
    }
}
exports.resolvePkgExport = resolvePkgExport;
const { pkg: cwdPkg, pkgDir: cwdPkgBaseDir } = (0, file_utils_1.findUpPkgJson)(process.cwd());
const cwdPkgName = (cwdPkg === null || cwdPkg === void 0 ? void 0 : cwdPkg.name) || '';
function packageFilter({ pkg, pkgDir, resolveType, noCjsFallback, subPath, }) {
    let resolvedPath = resolvePkgExport({
        pkg,
        resolveType,
        subPath,
    });
    if (!resolvedPath && resolveType === 'mjs' && !noCjsFallback) {
        resolvedPath = resolvePkgExport({
            pkg,
            resolveType: 'cjs',
            subPath,
        });
    }
    // if we can't find types, we can try to find a `.d.ts` file
    if (!resolvedPath && resolveType === 'types') {
        const typesToTry = ['mjs', 'cjs'];
        typesToTry.forEach((typeToTry) => {
            if (resolvedPath)
                return; // if we already found it, don't try the others
            const thisResolvedPath = resolvePkgExport({
                pkg,
                resolveType: typeToTry,
                subPath,
            });
            if (thisResolvedPath) {
                // can be a `.mjs` or `.js`
                const { dir, name, ext } = (0, node_path_1.parse)(thisResolvedPath);
                // turn into `.d.mts` or `.d.ts`
                const dtsPath = (0, node_path_1.format)({
                    dir,
                    name,
                    ext: convertJsExtensionToDts(ext),
                });
                const dtsFullPath = (0, node_path_1.join)(pkgDir, dtsPath);
                if ((0, file_utils_1.existsSync)(dtsFullPath)) {
                    resolvedPath = dtsPath;
                }
            }
        });
    }
    if (
    // last chance - nothing so far
    !resolvedPath &&
        // if there was an `exports` object then `resolvePkgExport` should of handled it
        !pkg.exports &&
        // is package sub-path
        subPath !== '.' &&
        // not types - those would never be resolved directly
        resolveType !== 'types' &&
        // actually exists!
        (0, file_utils_1.existsSync)((0, node_path_1.join)(pkgDir, subPath))) {
        return subPath;
    }
    return resolvedPath;
}
function resolvePathSyncUncached({ path, resolveFromDir = process.cwd(), resolveType = 'mjs', noCjsFallback = false, }) {
    let type = 'unknown';
    function resolveIt(pathOrPkg, opt) {
        let thePathOrPkg = pathOrPkg;
        let subPath = '.'; // `.` is the default sub-path
        const result = getPathType(pathOrPkg);
        if (result.type === 'package-sub-path') {
            thePathOrPkg = result.pkgName;
            subPath = result.subPath;
        }
        return resolve_1.default.sync(thePathOrPkg, {
            ...opt,
            basedir: resolveFromDir,
            // note that 2nd param differs from async version
            packageFilter(aPkg, pkgDir) {
                const main = packageFilter({
                    pkg: aPkg,
                    pkgDir,
                    resolveType,
                    noCjsFallback,
                    subPath,
                });
                return {
                    ...aPkg,
                    main,
                };
            },
        });
    }
    if ((0, node_path_1.isAbsolute)(path)) {
        type = 'absolute';
        try {
            const absolutePath = resolveIt(path);
            return {
                originalPath: path,
                absolutePath,
                exists: true,
                type,
            };
        }
        catch (e) {
            return {
                exists: false,
                originalPath: path,
                type: 'absolute',
                absolutePath: '',
            };
        }
    }
    if (path.startsWith('.')) {
        try {
            const absolutePath = resolveIt(path);
            return {
                originalPath: path,
                absolutePath,
                exists: true,
                type: 'relative',
            };
        }
        catch (e) {
            return {
                exists: false,
                originalPath: path,
                type: 'relative',
                absolutePath: '',
            };
        }
    }
    let absolutePath;
    try {
        type = 'package';
        if (path === cwdPkgName) {
            // if we are passing in 'my-ds' and that's the name of our NPM package, normal `require.resolve()` won't work since it's not in a 'node_modules' directory, so we pass in the path to the directory with the package.json instead
            absolutePath = resolveIt(cwdPkgBaseDir);
        }
        else if (path.startsWith(`${cwdPkgName}/`)) {
            absolutePath = resolveIt(path.replace(cwdPkgName, cwdPkgBaseDir));
        }
        else {
            absolutePath = resolveIt(path);
        }
        if (absolutePath) {
            return {
                originalPath: path,
                exists: true,
                absolutePath,
                type,
            };
        }
    }
    catch (e) {
        return {
            exists: false,
            originalPath: path,
            type: 'package',
            absolutePath: '',
        };
    }
    return {
        exists: false,
        originalPath: path,
        type: 'package',
        absolutePath: '',
    };
}
const resolvePathSyncCache = new Map();
/**
 * @deprecated use `resolvePath` instead since it's async
 */
function resolvePathSync(args) {
    const key = JSON.stringify(args);
    try {
        if (resolvePathSyncCache.has(key)) {
            return resolvePathSyncCache.get(key);
        }
        const result = resolvePathSyncUncached(args);
        if (result.exists) {
            // only cache successes, fails may try again later
            resolvePathSyncCache.set(key, result);
            // clear the cache after 10 seconds
            setTimeout(() => resolvePathSyncCache.delete(key), resolvePathCacheLife);
        }
        return result;
    }
    catch (e) {
        resolvePathSyncCache.delete(key);
        throw e;
    }
}
exports.resolvePathSync = resolvePathSync;
async function resolvePathUncached({ path, resolveFromDir = process.cwd(), resolveType = 'mjs', noCjsFallback = false, }) {
    let type = 'unknown';
    function resolveIt(pathOrPkg, opt) {
        return new Promise((resolvePromise, reject) => {
            let thePathOrPkg = pathOrPkg;
            let subPath = '.'; // `.` is the default sub-path
            const result = getPathType(pathOrPkg);
            if (result.type === 'package-sub-path') {
                thePathOrPkg = result.pkgName;
                subPath = result.subPath;
            }
            (0, resolve_1.default)(thePathOrPkg, {
                ...opt,
                basedir: resolveFromDir,
                // note that 2nd param differs from sync version
                packageFilter(aPkg, pkgJsonPath) {
                    const pkgDir = (0, node_path_1.dirname)(pkgJsonPath);
                    const main = packageFilter({
                        pkg: aPkg,
                        pkgDir,
                        resolveType,
                        noCjsFallback,
                        subPath,
                    });
                    return {
                        ...aPkg,
                        main,
                    };
                },
            }, (err, resolvedPath) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolvePromise(resolvedPath);
                }
            });
        });
    }
    if ((0, node_path_1.isAbsolute)(path)) {
        type = 'absolute';
        try {
            const absolutePath = await resolveIt(path);
            return {
                originalPath: path,
                absolutePath,
                exists: true,
                type,
            };
        }
        catch (e) {
            return {
                exists: false,
                originalPath: path,
                type: 'absolute',
                absolutePath: '',
            };
        }
    }
    if (path.startsWith('.')) {
        try {
            const absolutePath = await resolveIt(path);
            return {
                originalPath: path,
                absolutePath,
                exists: true,
                type: 'relative',
            };
        }
        catch (e) {
            return {
                exists: false,
                originalPath: path,
                type: 'relative',
                absolutePath: '',
            };
        }
    }
    let absolutePath;
    try {
        type = 'package';
        if (path === cwdPkgName) {
            // if we are passing in 'my-ds' and that's the name of our NPM package, normal `require.resolve()` won't work since it's not in a 'node_modules' directory, so we pass in the path to the directory with the package.json instead
            absolutePath = await resolveIt(cwdPkgBaseDir);
        }
        else if (path.startsWith(`${cwdPkgName}/`)) {
            absolutePath = await resolveIt(path.replace(cwdPkgName, cwdPkgBaseDir));
        }
        else {
            absolutePath = await resolveIt(path);
        }
        if (absolutePath) {
            return {
                originalPath: path,
                exists: true,
                absolutePath,
                type,
            };
        }
    }
    catch (e) {
        return {
            exists: false,
            originalPath: path,
            type: 'package',
            absolutePath: '',
        };
    }
    return {
        exists: false,
        originalPath: path,
        type: 'package',
        absolutePath: '',
    };
}
const resolvePathCache = new Map();
async function resolvePath(args) {
    const key = JSON.stringify(args);
    if (!resolvePathCache.has(key)) {
        const result = resolvePathUncached(args);
        resolvePathCache.set(key, result);
        setTimeout(() => resolvePathCache.delete(key), resolvePathCacheLife);
        result
            .then(({ exists }) => {
            if (!exists) {
                resolvePathCache.delete(key);
            }
        })
            .catch(() => {
            resolvePathCache.delete(key);
        });
    }
    return resolvePathCache.get(key);
}
exports.resolvePath = resolvePath;
//# sourceMappingURL=resolve.js.map